import IActor from "./IActor";
import IMailbox from "./IMailbox";
import { MultiMessageHandler } from "./IMultiMessageHandler";
import IErrorHandler from "./IErrorHandler";
/**
 * Processes one message at a time.
 * @template TMessages The handled message types.
 */
export default class Actor<TMessages> implements IActor<TMessages> {
    private readonly multiMessageHandler;
    private readonly errorHandler;
    private running;
    private readonly mailbox;
    /**
     * @param Mailbox The constructor for the mailbox to use.
     * @param multiMessageHandler The handlers for messages processed by the actor.
     * @param errorHandler The handler for errors raised while processing
     * messages.
     */
    constructor(Mailbox: {
        new (): IMailbox<TMessages>;
    }, multiMessageHandler: MultiMessageHandler<TMessages>, errorHandler: IErrorHandler);
    /**
     * @inheritdoc
     */
    tell<TKey extends keyof TMessages>(key: TKey, value: TMessages[TKey]): void;
    private done;
}
