"use strict";
exports.__esModule = true;
/**
 * Processes one message at a time.
 * @template TMessages The handled message types.
 */
var Actor = /** @class */ (function () {
    /**
     * @param Mailbox The constructor for the mailbox to use.
     * @param multiMessageHandler The handlers for messages processed by the actor.
     * @param errorHandler The handler for errors raised while processing
     * messages.
     */
    function Actor(Mailbox, multiMessageHandler, errorHandler) {
        this.multiMessageHandler = multiMessageHandler;
        this.errorHandler = errorHandler;
        this.running = false;
        this.mailbox = new Mailbox();
    }
    /**
     * @inheritdoc
     */
    Actor.prototype.tell = function (key, value) {
        var _this = this;
        if (this.running) {
            this.mailbox.push({ key: key, value: value });
        }
        else {
            this.running = true;
            this.multiMessageHandler[key](this, value).then(function () { return _this.done(); }, function (reason) {
                _this.errorHandler(reason);
                _this.done();
            });
        }
    };
    Actor.prototype.done = function () {
        this.running = false;
        var nextMessage = this.mailbox.shift();
        if (nextMessage) {
            this.tell(nextMessage.key, nextMessage.value);
        }
    };
    return Actor;
}());
exports["default"] = Actor;
//# sourceMappingURL=Actor.js.map