"use strict";
exports.__esModule = true;
require("jasmine");
var Core_1 = require("./Core");
var core;
var pluginsInstances;
var pluginsPush;
var pluginsForEach;
var stateInstances;
var stateInstance;
var stateInitial;
var stateGet;
var stateSet;
var pluginActorInstances;
var pluginActorInstance;
var pluginActorMailbox;
var pluginActorMultiMessageHandler;
var pluginActorErrorHandler;
var pluginActorTell;
var loggerActorInstances;
var loggerActorInstance;
var loggerActorMailbox;
var loggerActorMultiMessageHandler;
var loggerActorErrorHandler;
var loggerActorTell;
var LoggerProxy = /** @class */ (function () {
    function LoggerProxy(logger, name) {
        this.logger = logger;
        this.name = name;
        this.tell = jasmine.createSpy();
        loggerProxyInstances.push(this);
    }
    return LoggerProxy;
}());
var loggerProxyInstances;
var errorHandler;
var application;
var applicationInitial;
var applicationApply;
var loggerVerbose;
var loggerInformation;
var loggerWarning;
var loggerError;
var logger;
beforeEach(function () {
    pluginsInstances = 0;
    pluginsPush = jasmine.createSpy();
    pluginsForEach = jasmine.createSpy();
    var PluginsStatelessSet = /** @class */ (function () {
        function PluginsStatelessSet() {
            this.push = pluginsPush;
            this.forEach = pluginsForEach;
            pluginsInstances++;
        }
        return PluginsStatelessSet;
    }());
    stateInstances = 0;
    stateGet = jasmine.createSpy();
    stateSet = jasmine.createSpy();
    var StateContainer = /** @class */ (function () {
        function StateContainer(initial) {
            this.get = stateGet;
            this.set = stateSet;
            stateInstances++;
            stateInstance = this;
            stateInitial = initial;
        }
        return StateContainer;
    }());
    pluginActorInstances = 0;
    pluginActorTell = jasmine.createSpy();
    var PluginActor = /** @class */ (function () {
        function PluginActor(Mailbox, multiMessageHandler, errorHandler) {
            this.tell = pluginActorTell;
            pluginActorInstances++;
            pluginActorInstance = this;
            pluginActorMailbox = Mailbox;
            pluginActorMultiMessageHandler = multiMessageHandler;
            pluginActorErrorHandler = errorHandler;
        }
        return PluginActor;
    }());
    loggerActorInstances = 0;
    loggerActorTell = jasmine.createSpy();
    var LoggerActor = /** @class */ (function () {
        function LoggerActor(Mailbox, multiMessageHandler, errorHandler) {
            this.tell = loggerActorTell;
            loggerActorInstances++;
            loggerActorInstance = this;
            loggerActorMailbox = Mailbox;
            loggerActorMultiMessageHandler = multiMessageHandler;
            loggerActorErrorHandler = errorHandler;
        }
        return LoggerActor;
    }());
    loggerProxyInstances = [];
    errorHandler = jasmine.createSpy();
    applicationInitial = jasmine.createSpy();
    applicationInitial.and.returnValue({
        stateContent: "Test Initial State Content"
    });
    applicationApply = jasmine.createSpy();
    application = {
        title: "Test Application Title",
        initial: applicationInitial,
        apply: applicationApply,
        differentiatedBy: "this"
    };
    loggerVerbose = jasmine.createSpy();
    loggerInformation = jasmine.createSpy();
    loggerWarning = jasmine.createSpy();
    loggerError = jasmine.createSpy();
    logger = {
        verbose: loggerVerbose,
        information: loggerInformation,
        warning: loggerWarning,
        error: loggerError
    };
    core = new Core_1["default"](PluginsStatelessSet, StateContainer, PluginActor, LoggerActor, LoggerProxy, errorHandler, application, logger);
});
it("creates one stateless set of plugins", function () { return expect(pluginsInstances).toEqual(1); });
it("does not push to the stateless set of plugins", function () { return expect(pluginsPush).not.toHaveBeenCalled(); });
it("does not iterate over the stateless set of plugins", function () { return expect(pluginsForEach).not.toHaveBeenCalled(); });
it("creates one state container", function () { return expect(stateInstances).toEqual(1); });
it("creates the state container with the application's initial state", function () { return expect(stateInitial).toEqual({
    stateContent: "Test Initial State Content"
}); });
it("does not get the state", function () { return expect(stateGet).not.toHaveBeenCalled(); });
it("does not set the state", function () { return expect(stateSet).not.toHaveBeenCalled(); });
it("does not create any plugin actors", function () { return expect(pluginActorInstances).toEqual(0); });
it("creates one logger actor", function () { return expect(loggerActorInstances).toEqual(1); });
it("creates the logger actor with an array as the mailbox", function () { return expect(loggerActorMailbox).toBe(Array); });
it("creates the logger actor with the plugin's message handler", function () { return expect(loggerActorMultiMessageHandler).toBe(logger); });
it("creates the logger actor with the error handler", function () { return expect(loggerActorErrorHandler).toBe(errorHandler); });
it("does not tell the created logger actor", function () { return expect(loggerActorTell).not.toHaveBeenCalled(); });
it("creates one logger proxy", function () { return expect(loggerProxyInstances.length).toEqual(1); });
it("creates the logger proxy using a name of \"Core\"", function () { return expect(loggerProxyInstances[0].name).toEqual("Core"); });
it("creates the logger proxy using the logger actor", function () { return expect(loggerProxyInstances[0].logger).toBe(loggerActorInstance); });
it("does not tell the logger proxy", function () { return expect(loggerProxyInstances[0].tell).not.toHaveBeenCalled(); });
it("does not handle any errors", function () { return expect(errorHandler).not.toHaveBeenCalled(); });
it("gets the initial state of the application", function () { return expect(applicationInitial).toHaveBeenCalledTimes(1); });
it("does not apply any events", function () { return expect(applicationApply).not.toHaveBeenCalled(); });
it("does not directly invoke the logger's \"verbose\" message handler", function () { return expect(loggerVerbose).not.toHaveBeenCalled(); });
it("does not directly invoke the logger's \"information\" message handler", function () { return expect(loggerInformation).not.toHaveBeenCalled(); });
it("does not directly invoke the logger's \"warning\" message handler", function () { return expect(loggerWarning).not.toHaveBeenCalled(); });
it("does not directly invoke the logger's \"verbose\" message handler", function () { return expect(loggerError).not.toHaveBeenCalled(); });
describe("install", function () {
    var receivedByTell;
    var receivedBy;
    var pluginInstalled;
    var pluginStateChanged;
    var plugin;
    beforeEach(function () {
        receivedByTell = jasmine.createSpy();
        receivedBy = {
            tell: receivedByTell
        };
        pluginInstalled = jasmine.createSpy();
        pluginStateChanged = jasmine.createSpy();
        plugin = {
            installed: pluginInstalled,
            stateChanged: pluginStateChanged,
            name: "Test Plugin Name"
        };
        core.install(receivedBy, {
            plugin: plugin
        });
    });
    it("does not create further stateless sets of plugins", function () { return expect(pluginsInstances).toEqual(1); });
    it("pushes one item to the stateless set of plugins", function () { return expect(pluginsPush).toHaveBeenCalledTimes(1); });
    it("pushes the created plugin actor to the stateless set of plugins", function () { return expect(pluginsPush.calls.argsFor(0)[0]).toBe(pluginActorInstance); });
    it("does not iterate over the stateless set of plugins", function () { return expect(pluginsForEach).not.toHaveBeenCalled(); });
    it("does not create further state containers", function () { return expect(stateInstances).toEqual(1); });
    it("does not get the state", function () { return expect(stateGet).not.toHaveBeenCalled(); });
    it("does not set the state", function () { return expect(stateSet).not.toHaveBeenCalled(); });
    it("creates one plugin actor", function () { return expect(pluginActorInstances).toEqual(1); });
    it("creates the plugin actor with an array as the mailbox", function () { return expect(pluginActorMailbox).toBe(Array); });
    it("creates the plugin actor with the plugin's message handler", function () { return expect(pluginActorMultiMessageHandler).toBe(plugin); });
    it("creates the plugin actor with the error handler", function () { return expect(pluginActorErrorHandler).toBe(errorHandler); });
    it("tells the created plugin actor once", function () { return expect(pluginActorTell).toHaveBeenCalledTimes(1); });
    it("tells the created plugin actor it has been installed", function () { return expect(pluginActorTell).toHaveBeenCalledWith("installed", {
        core: receivedBy,
        application: application,
        state: stateInstance,
        logger: loggerProxyInstances[1]
    }); });
    it("does not create further logger actors", function () { return expect(loggerActorInstances).toEqual(1); });
    it("does not tell the logger actor", function () { return expect(loggerActorTell).not.toHaveBeenCalled(); });
    it("tells the core logger proxy once", function () { return expect(loggerProxyInstances[0].tell).toHaveBeenCalledTimes(1); });
    it("tells the core logger proxy to log that a plugin has been installed", function () { return expect(loggerProxyInstances[0].tell).toHaveBeenCalledWith("information", "Plugin \"Test Plugin Name\" has been installed."); });
    it("creates another logger proxy", function () { return expect(loggerProxyInstances.length).toEqual(2); });
    it("creates the logger proxy using the plugin's name", function () { return expect(loggerProxyInstances[1].name).toEqual("Test Plugin Name"); });
    it("creates the logger proxy using the logger actor", function () { return expect(loggerProxyInstances[1].logger).toBe(loggerActorInstance); });
    it("does not tell the logger proxy", function () { return expect(loggerProxyInstances[1].tell).not.toHaveBeenCalled(); });
    it("does not handle any errors", function () { return expect(errorHandler).not.toHaveBeenCalled(); });
    it("does not get the initial state of the application again", function () { return expect(applicationInitial).toHaveBeenCalledTimes(1); });
    it("does not apply any events", function () { return expect(applicationApply).not.toHaveBeenCalled(); });
    it("does not tell itself", function () { return expect(receivedByTell).not.toHaveBeenCalled(); });
    it("does not directly invoke the plugin's \"installed\" message handler", function () { return expect(pluginInstalled).not.toHaveBeenCalled(); });
    it("does not directly invoke the plugin's \"stateChanged\" message handler", function () { return expect(pluginStateChanged).not.toHaveBeenCalled(); });
    it("does not directly invoke the logger's \"verbose\" message handler", function () { return expect(loggerVerbose).not.toHaveBeenCalled(); });
    it("does not directly invoke the logger's \"information\" message handler", function () { return expect(loggerInformation).not.toHaveBeenCalled(); });
    it("does not directly invoke the logger's \"warning\" message handler", function () { return expect(loggerWarning).not.toHaveBeenCalled(); });
    it("does not directly invoke the logger's \"verbose\" message handler", function () { return expect(loggerError).not.toHaveBeenCalled(); });
});
describe("replaceState", function () {
    var receivedByTell;
    var receivedBy;
    beforeEach(function () {
        receivedByTell = jasmine.createSpy();
        receivedBy = {
            tell: receivedByTell
        };
        core.replaceState(receivedBy, {
            state: {
                stateContent: "Test Replacing State Content"
            }
        });
    });
    it("does not create further stateless sets of plugins", function () { return expect(pluginsInstances).toEqual(1); });
    it("does not push to the stateless set of plugins", function () { return expect(pluginsPush).not.toHaveBeenCalled(); });
    it("iterates over the stateless set of plugins once", function () { return expect(pluginsForEach).toHaveBeenCalledTimes(1); });
    it("does not create further state containers", function () { return expect(stateInstances).toEqual(1); });
    it("does not get the state", function () { return expect(stateGet).not.toHaveBeenCalled(); });
    it("sets the state once", function () { return expect(stateSet).toHaveBeenCalledTimes(1); });
    it("sets the state to the replacing state", function () { return expect(stateSet).toHaveBeenCalledWith({
        stateContent: "Test Replacing State Content"
    }); });
    it("set the state before iterating over the plugins", function () { return expect(stateSet).toHaveBeenCalledBefore(pluginsForEach); });
    it("does not create any plugin actors", function () { return expect(pluginActorInstances).toEqual(0); });
    it("does not create further logger actors", function () { return expect(loggerActorInstances).toEqual(1); });
    it("does not tell the logger actor", function () { return expect(loggerActorTell).not.toHaveBeenCalled(); });
    it("tells the core logger proxy once", function () { return expect(loggerProxyInstances[0].tell).toHaveBeenCalledTimes(1); });
    it("tells the logger actor to log that a state has been replaced", function () { return expect(loggerProxyInstances[0].tell).toHaveBeenCalledWith("information", "Application state has been replaced."); });
    it("does not handle any errors", function () { return expect(errorHandler).not.toHaveBeenCalled(); });
    it("does not get the initial state of the application again", function () { return expect(applicationInitial).toHaveBeenCalledTimes(1); });
    it("does not apply any events", function () { return expect(applicationApply).not.toHaveBeenCalled(); });
    it("does not tell itself", function () { return expect(receivedByTell).not.toHaveBeenCalled(); });
    it("does not directly invoke the logger's \"verbose\" message handler", function () { return expect(loggerVerbose).not.toHaveBeenCalled(); });
    it("does not directly invoke the logger's \"information\" message handler", function () { return expect(loggerInformation).not.toHaveBeenCalled(); });
    it("does not directly invoke the logger's \"warning\" message handler", function () { return expect(loggerWarning).not.toHaveBeenCalled(); });
    it("does not directly invoke the logger's \"verbose\" message handler", function () { return expect(loggerError).not.toHaveBeenCalled(); });
    describe("the callback given to the stateless set of plugins", function () {
        var pluginTell;
        beforeEach(function () {
            receivedByTell = jasmine.createSpy();
            receivedBy = {
                tell: receivedByTell
            };
            pluginTell = jasmine.createSpy();
            pluginsForEach.calls.argsFor(0)[0]({
                tell: pluginTell
            });
        });
        it("does not create further stateless sets of plugins", function () { return expect(pluginsInstances).toEqual(1); });
        it("does not push to the stateless set of plugins", function () { return expect(pluginsPush).not.toHaveBeenCalled(); });
        it("does not iterate over the stateless set of plugins again", function () { return expect(pluginsForEach).toHaveBeenCalledTimes(1); });
        it("does not create further state containers", function () { return expect(stateInstances).toEqual(1); });
        it("does not get the state", function () { return expect(stateGet).not.toHaveBeenCalled(); });
        it("does not set the state again", function () { return expect(stateSet).toHaveBeenCalledTimes(1); });
        it("does not create any plugin actors", function () { return expect(pluginActorInstances).toEqual(0); });
        it("does not create further logger actors", function () { return expect(loggerActorInstances).toEqual(1); });
        it("does not tell the logger actor", function () { return expect(loggerActorTell).not.toHaveBeenCalled(); });
        it("does not tell the core logger proxy again", function () { return expect(loggerProxyInstances[0].tell).toHaveBeenCalledTimes(1); });
        it("does not handle any errors", function () { return expect(errorHandler).not.toHaveBeenCalled(); });
        it("does not get the initial state of the application again", function () { return expect(applicationInitial).toHaveBeenCalledTimes(1); });
        it("does not apply any events", function () { return expect(applicationApply).not.toHaveBeenCalled(); });
        it("does not tell itself", function () { return expect(receivedByTell).not.toHaveBeenCalled(); });
        it("tells the plugin once", function () { return expect(pluginTell).toHaveBeenCalledTimes(1); });
        it("tells the plugin that state changed", function () { return expect(pluginTell).toHaveBeenCalledWith("stateChanged", {
            state: {
                stateContent: "Test Replacing State Content"
            },
            event: null
        }); });
        it("does not directly invoke the logger's \"verbose\" message handler", function () { return expect(loggerVerbose).not.toHaveBeenCalled(); });
        it("does not directly invoke the logger's \"information\" message handler", function () { return expect(loggerInformation).not.toHaveBeenCalled(); });
        it("does not directly invoke the logger's \"warning\" message handler", function () { return expect(loggerWarning).not.toHaveBeenCalled(); });
        it("does not directly invoke the logger's \"verbose\" message handler", function () { return expect(loggerError).not.toHaveBeenCalled(); });
    });
});
//# sourceMappingURL=Core.Tests.js.map