import IJsonObject from "../../IJsonObject";
import IStatelessSet from "../../State/IStatelessSet";
import IStateContainer from "../../State/IStateContainer";
import { MultiMessageHandler } from "../../Actors/IMultiMessageHandler";
import IErrorHandler from "../../Actors/IErrorHandler";
import IMailbox from "../../Actors/IMailbox";
import IActor from "../../Actors/IActor";
import ILogMessages from "../../Logging/ILogMessages";
import ILoggerProxy from "../../Logging/ILoggerProxy";
import IApplication from "../IApplication";
import ICoreMessages from "./ICoreMessages";
import IPlugin from "./IPlugin";
import IPluginMessages from "./IPluginMessages";
/**
 * Hosts an application, performing all "plumbing".
 * @template TState The JSON-serializable type of application state.
 * @template TEvent The JSON-serializable type of changes to application state.
 * @template TApplication The application type hosted.
 */
export default class Core<TState extends IJsonObject, TEvent extends IJsonObject, TApplication extends IApplication<TState, TEvent>> implements MultiMessageHandler<ICoreMessages<TState, TEvent, TApplication>> {
    private readonly PluginActor;
    private readonly LoggerProxy;
    private readonly errorHandler;
    private readonly application;
    private readonly plugins;
    private readonly state;
    private readonly logger;
    private readonly loggerProxy;
    /**
     * @param PluginsStatelessSet The constructor for the set of installed
     * plugins.
     * @param errorHandler The error handler for plugins to use.
     * @param application The application hosted.
     * @param logger The logger to use.
     */
    constructor(PluginsStatelessSet: {
        new (): IStatelessSet<IActor<IPluginMessages<TState, TEvent, TApplication>>>;
    }, StateContainer: {
        new (state: TState): IStateContainer<TState>;
    }, PluginActor: {
        new (Mailbox: {
            new (): IMailbox<IPluginMessages<TState, TEvent, TApplication>>;
        }, multiMessageHandler: MultiMessageHandler<IPluginMessages<TState, TEvent, TApplication>>, errorHandler: IErrorHandler): IActor<IPluginMessages<TState, TEvent, TApplication>>;
    }, LoggerActor: {
        new (Mailbox: {
            new (): IMailbox<ILogMessages>;
        }, multiMessageHandler: MultiMessageHandler<ILogMessages>, errorHandler: IErrorHandler): IActor<ILogMessages>;
    }, LoggerProxy: {
        new (logger: IActor<ILogMessages>, name: string): ILoggerProxy;
    }, errorHandler: IErrorHandler, application: TApplication, logger: MultiMessageHandler<ILogMessages>);
    /**
     * @inheritdoc
     */
    install(receivedBy: IActor<ICoreMessages<TState, TEvent, TApplication>>, message: {
        readonly plugin: IPlugin<TState, TEvent, TApplication>;
    }): Promise<void>;
    /**
     * @inheritdoc
     */
    replaceState(receivedBy: IActor<ICoreMessages<TState, TEvent, TApplication>>, message: {
        readonly state: TState;
    }): Promise<void>;
}
