"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
/**
 * Hosts an application, performing all "plumbing".
 * @template TState The JSON-serializable type of application state.
 * @template TEvent The JSON-serializable type of changes to application state.
 * @template TApplication The application type hosted.
 */
var Core = /** @class */ (function () {
    /**
     * @param PluginsStatelessSet The constructor for the set of installed
     * plugins.
     * @param errorHandler The error handler for plugins to use.
     * @param application The application hosted.
     * @param logger The logger to use.
     */
    function Core(PluginsStatelessSet, StateContainer, PluginActor, LoggerActor, LoggerProxy, errorHandler, application, logger) {
        this.PluginActor = PluginActor;
        this.LoggerProxy = LoggerProxy;
        this.errorHandler = errorHandler;
        this.application = application;
        this.plugins = new PluginsStatelessSet();
        this.state = new StateContainer(application.initial());
        this.logger = new LoggerActor(Array, logger, errorHandler);
        this.loggerProxy = new LoggerProxy(this.logger, "Core");
    }
    /**
     * @inheritdoc
     */
    Core.prototype.install = function (receivedBy, message) {
        return __awaiter(this, void 0, void 0, function () {
            var actor;
            return __generator(this, function (_a) {
                actor = new this.PluginActor(Array, message.plugin, this.errorHandler);
                this.plugins.push(actor);
                actor.tell("installed", {
                    core: receivedBy,
                    application: this.application,
                    state: this.state,
                    logger: new this.LoggerProxy(this.logger, message.plugin.name)
                });
                this.loggerProxy.tell("information", "Plugin \"" + message.plugin.name + "\" has been installed.");
                return [2 /*return*/];
            });
        });
    };
    /**
     * @inheritdoc
     */
    Core.prototype.replaceState = function (receivedBy, message) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                this.state.set(message.state);
                this.plugins.forEach(function (plugin) { return plugin.tell("stateChanged", {
                    state: message.state,
                    event: null
                }); });
                this.loggerProxy.tell("information", "Application state has been replaced.");
                return [2 /*return*/];
            });
        });
    };
    return Core;
}());
exports["default"] = Core;
//# sourceMappingURL=Core.js.map