/// <reference types="node" />
import IJsonObject from "../../../IJsonObject";
import IReadOnlyStateContainer from "../../../State/IReadOnlyStateContainer";
import ILoggerProxy from "../../../Logging/ILoggerProxy";
import IActor from "../../../Actors/IActor";
import IApplication from "../../IApplication";
import IPluginMessages from "../IPluginMessages";
import ICoreMessages from "../ICoreMessages";
import IPlugin from "../IPlugin";
/**
 * Persists the latest state to a single file.
 * @template TState The JSON-serializable type of application state.
 * @template TEvent The JSON-serializable type of changes to application state.
 * @template TApplication The application type interfaced with.
 */
export default class FilePlugin<TState extends IJsonObject, TEvent extends IJsonObject, TApplication extends IApplication<TState, TEvent>> implements IPlugin<TState, TEvent, TApplication> {
    private readonly pathDirname;
    private readonly filename;
    private readonly fsReadFile;
    private readonly mkdirp;
    private readonly fsWriteFile;
    private readonly fsRename;
    private readonly tempFilename;
    /**
     * @param fsReadFile Enables dependency injection in tests.
     * @param pathDirname Enables dependency injection in tests.
     * @param mkdirp Enables dependency injection in tests.
     * @param fsWriteFile Enables dependency injection in tests.
     * @param fsRename Enables dependency injection in tests.
     * @param filename The path to the file to persist to.
     */
    constructor(fsReadFile: (path: string, options: {
        encoding: string;
    }, callback: (err: NodeJS.ErrnoException, data: string) => void) => void, pathDirname: (p: string) => string, mkdirp: (p: string, cb: (err: Error | null) => void) => void, fsWriteFile: (path: string, data: string, callback: (err: NodeJS.ErrnoException) => void) => void, fsRename: (oldPath: string, newPath: string, callback: (err: NodeJS.ErrnoException) => void) => void, filename: string);
    /**
    * @inheritdoc
    */
    readonly name = "File";
    /**
    * @inheritdoc
    */
    installed(receivedBy: IActor<IPluginMessages<TState, TEvent, TApplication>>, message: {
        readonly core: IActor<ICoreMessages<TState, TEvent, TApplication>>;
        readonly application: TApplication;
        readonly state: IReadOnlyStateContainer<TState>;
        readonly logger: ILoggerProxy;
    }): Promise<void>;
    /**
    * @inheritdoc
    */
    stateChanged(receivedBy: IActor<IPluginMessages<TState, TEvent, TApplication>>, message: {
        readonly state: TState;
        readonly event: null | TEvent;
    }): Promise<void>;
}
