"use strict";
exports.__esModule = true;
require("jasmine");
var StateSet_1 = require("./StateSet");
var State;
(function (State) {
    State[State["EnsuresNotJustCheckingForTruthiness"] = 0] = "EnsuresNotJustCheckingForTruthiness";
    State[State["SwapA"] = 1] = "SwapA";
    State[State["ChangedAFrom"] = 2] = "ChangedAFrom";
    State[State["D"] = 3] = "D";
    State[State["E"] = 4] = "E";
    State[State["F"] = 5] = "F";
    State[State["Unchanged"] = 6] = "Unchanged";
    State[State["SwapB"] = 7] = "SwapB";
    State[State["Removed"] = 8] = "Removed";
    State[State["ChangedATo"] = 9] = "ChangedATo";
    State[State["K"] = 10] = "K";
    State[State["L"] = 11] = "L";
    State[State["M"] = 12] = "M";
    State[State["N"] = 13] = "N";
})(State || (State = {}));
var stateSet;
beforeEach(function () { return stateSet = new StateSet_1["default"](); });
describe("add", function () {
    beforeEach(function () {
        stateSet.add(State.ChangedAFrom);
        stateSet.add(State.Unchanged);
        stateSet.add(State.SwapA);
        stateSet.add(State.Removed);
        stateSet.add(State.SwapB);
        stateSet.add(State.F);
    });
    it("does nothing", function () { });
    describe("process", function () {
        var handler;
        beforeEach(function () {
            handler = jasmine.createSpy();
            handler.and.callFake(function (state) {
                switch (state) {
                    case State.ChangedAFrom:
                        return State.ChangedATo;
                    case State.Unchanged:
                        return State.Unchanged;
                    case State.SwapA:
                        return State.SwapB;
                    case State.Removed:
                        return null;
                    case State.SwapB:
                        return State.SwapA;
                    case State.F:
                        return State.EnsuresNotJustCheckingForTruthiness;
                    default:
                        fail(state);
                        return null;
                }
            });
            stateSet.process(handler);
        });
        it("executes the handler once per added state", function () { return expect(handler).toHaveBeenCalledTimes(6); });
        it("executes the handler with the first state", function () { return expect(handler).toHaveBeenCalledWith(State.ChangedAFrom); });
        it("executes the handler with the second state", function () { return expect(handler).toHaveBeenCalledWith(State.Unchanged); });
        it("executes the handler with the third state", function () { return expect(handler).toHaveBeenCalledWith(State.SwapA); });
        it("executes the handler with the fourth state", function () { return expect(handler).toHaveBeenCalledWith(State.Removed); });
        it("executes the handler with the fifth state", function () { return expect(handler).toHaveBeenCalledWith(State.SwapB); });
        it("executes the handler with the sixth state", function () { return expect(handler).toHaveBeenCalledWith(State.F); });
        describe("add", function () {
            beforeEach(function () {
                handler.calls.reset();
                stateSet.add(State.E);
                stateSet.add(State.M);
                stateSet.add(State.L);
            });
            it("does not execute the original handler", function () { return expect(handler).not.toHaveBeenCalled(); });
            describe("process", function () {
                var subsequentHandler;
                beforeEach(function () {
                    subsequentHandler = jasmine.createSpy();
                    subsequentHandler.and.returnValue(null);
                    stateSet.process(subsequentHandler);
                });
                it("does not execute the original handler", function () { return expect(handler).not.toHaveBeenCalled(); });
                it("executes the subsequent handler once per state which subsequently "
                    + " added or not removed", function () { return expect(subsequentHandler).toHaveBeenCalledTimes(8); });
                it("executes the subsequent handler with the first unremoved state", function () { return expect(subsequentHandler).toHaveBeenCalledWith(State.ChangedATo); });
                it("executes the subsequent handler with the second unremoved state", function () { return expect(subsequentHandler).toHaveBeenCalledWith(State.Unchanged); });
                it("executes the subsequent handler with the third unremoved state", function () { return expect(subsequentHandler).toHaveBeenCalledWith(State.SwapB); });
                it("executes the subsequent handler with the fourth unremoved state", function () { return expect(subsequentHandler).toHaveBeenCalledWith(State.SwapA); });
                it("executes the subsequent handler with the fifth unremoved state", function () { return expect(subsequentHandler)
                    .toHaveBeenCalledWith(State.EnsuresNotJustCheckingForTruthiness); });
                it("executes the subsequent handler with the first subsequently added "
                    + "state", function () { return expect(subsequentHandler).toHaveBeenCalledWith(State.E); });
                it("executes the subsequent handler with the second subsequently added "
                    + "state", function () { return expect(subsequentHandler).toHaveBeenCalledWith(State.M); });
                it("executes the subsequent handler with the third subsequently added "
                    + "state", function () { return expect(subsequentHandler).toHaveBeenCalledWith(State.L); });
            });
        });
        describe("process", function () {
            var subsequentHandler;
            beforeEach(function () {
                handler.calls.reset();
                subsequentHandler = jasmine.createSpy();
                subsequentHandler.and.returnValue(null);
                stateSet.process(subsequentHandler);
            });
            it("does not execute the original handler", function () { return expect(handler).not.toHaveBeenCalled(); });
            it("executes the subsequent handler once per state which was not removed", function () { return expect(subsequentHandler).toHaveBeenCalledTimes(5); });
            it("executes the subsequent handler with the first unremoved state", function () { return expect(subsequentHandler).toHaveBeenCalledWith(State.ChangedATo); });
            it("executes the subsequent handler with the second unremoved state", function () { return expect(subsequentHandler).toHaveBeenCalledWith(State.Unchanged); });
            it("executes the subsequent handler with the third unremoved state", function () { return expect(subsequentHandler).toHaveBeenCalledWith(State.SwapB); });
            it("executes the subsequent handler with the fourth unremoved state", function () { return expect(subsequentHandler).toHaveBeenCalledWith(State.SwapA); });
            it("executes the subsequent handler with the fifth unremoved state", function () { return expect(subsequentHandler)
                .toHaveBeenCalledWith(State.EnsuresNotJustCheckingForTruthiness); });
        });
    });
});
//# sourceMappingURL=StateSet.Tests.js.map